#include <bits/stdc++.h>

using namespace std;

typedef pair<int,int> Edge;
typedef pair<vector<Edge>, pair<int, int> > Graph;

inline void normalize(Edge &e) { if (e.first > e.second) swap(e.first, e.second); }
inline int randint(int f, int t) { return rand() % (t - f + 1) + f; }
inline bool randbit(void) { return rand() % 2; }
const char test_base[] = "p1-1data/";

/* Náhodně vrátí (uspořádanou) K-tici z {0, ..., N - 1} */
vector<int> pick(int K, int N)
{
	vector<int> v(N);
	for (int i = 0; i < N; i++)
		v[i] = i;

	random_shuffle(v.begin(), v.end());
	v.resize(K);
	return v;
}

/* Přečísluje vrcholy v jednotlivých grafech a vrátí jeden velký graf */
Graph multiplex(const vector<Graph> &Gr)
{
	int N = 0;
	vector<Edge> imp, unimp;
	for (auto G : Gr) {
		int k = G.second.first;
		for (int i = 0; i < (int)G.first.size(); i++) {
			auto e = G.first[i];
			e.first += N, e.second += N;
			((i < k) ? imp : unimp).push_back(e);
		}
		N += G.second.second;
	}

	Graph res;
	res.second.second = N;
	res.second.first = imp.size();
	for (auto e : imp)
		res.first.push_back(e);
	for (auto e : unimp)
		res.first.push_back(e);
	
	return res;
}

/* Zamíchá čísly vrcholů grafu a propermutuje jeho hrany */
void scramble(Graph &G)
{
	int N = G.second.second;
	vector<int> transl = pick(N, N);

	for (auto &e : G.first) {
		e = {transl[e.first], transl[e.second]};
		if (randbit())
			swap(e.first, e.second);
	}

	int K = G.second.first;
	random_shuffle(G.first.begin(), G.first.begin() + K);
	random_shuffle(G.first.begin() + K, G.first.end());
}

/* Dumpuje graf ve správném formátu do souboru */
void print(int test, int subtest, const vector<Graph> &Gr)
{
	Graph G = multiplex(Gr);
	scramble(G);
	int N = G.second.second, M = G.first.size(), K = G.second.first;
	char filename[100];
	sprintf(filename, "%s%02d%c.in", test_base, test, 'a' + subtest);
	FILE *o = fopen(filename, "w");
	fprintf(o, "%d %d %d\n", N, M, K);
	for (auto e : G.first)
		fprintf(o, "%d %d\n", e.first + 1, e.second + 1);
	fclose(o);
}

void print(int test, int subtest, const Graph &G) { return print(test, subtest, (vector<Graph>){G}); }

/* Vygeneruje strom, ve kterém mají vrcholy sudé stupně, pak pospojuje listy */
Graph gen_pseudotree_MeqK(int N, bool bad = false)
{
	Graph G;
	G.second.second = N;
	vector<int> degs = {0};
	int pos = 0;
	for (int i = 1; i < N; i++) {
		if (!degs[pos] && pos == i - 1)
			degs[pos] += 2;

		if (!degs[pos])
			pos++;

		degs[pos]--;
		G.first.push_back({pos, i});
		degs.push_back(1 + 2 * randint(1, 5));
	}

	if (degs[pos] % 2 == 0)
		pos++;

	for (int i = pos; i + 1 + bad < N; i += 2)
		G.first.push_back({i, i + 1});
	G.second.first = G.first.size();

	return G;
}

/* Přidá do grafu vrchol, jehož všechny hrany budou "důležité". Každá důležitá
 * hrana povede do jednoho vrcholu s odd == true */
Graph add_boss_vertex(vector<Edge> E, vector<bool> odd)
{
	int N = odd.size();
	Graph G;
	for (int i = 0; i < N; i++) {
		if (odd[i])
			G.first.push_back({i, N});
	}

	G.second.first = G.first.size();
	G.second.second = N + 1;
	for (auto e : E)
		G.first.push_back(e);

	return G;
}

/* TODO: generovat überhluboké a übernáhodné stromy */
vector<Edge> gen_tree(int N)
{
	vector<Edge> E;
	for (int i = 1; i < N; i++)
		E.push_back({randint(0, i - 1), i});
	return E;
}

/* Vyrobí strom a buď jednu z jeho hran zvolí jako důležitou (bad == true),
 * nebo přidá novou hranu mezi nějakými dvěma vrcholy */
Graph gen_K1_tree(int N, bool bad = false)
{
	Graph G;
	G.second = {1, N};
	G.first = gen_tree(N);

	random_shuffle(G.first.begin(), G.first.end());
	if (bad)
		return G;

	while (true) {
		Edge e = {randint(0, N - 1), randint(0, N - 1)};
		if (e.first > e.second) swap(e.first, e.second);
		if (e.first != e.second && !count(G.first.begin(), G.first.end(), e)) {
			G.first.insert(G.first.begin(), e);
			break;
		}
	}

	return G;
}

/* Nechá si vyrobit strom a pak nějaké z jeho vrcholů prohlásí za liché. */
Graph gen_from_tree(int N, bool bad = false)
{
	N--;
	int K = (randint(N / 8 + 1, N / 2 + 1) | 1) ^ (!bad);
	vector<int> chosen = pick(K, N);
	vector<bool> odd(N);
	for (auto a : chosen)
		odd[a] = true;

	vector<Edge> E = gen_tree(N);
	return add_boss_vertex(E, odd);
}


///////////////   vasek

Graph gen_vaskuv_graph(int N, int M){
	//graf, ktery shodi kvadraticke reseni, ktere xoruje cesticky a hleda je pomoci DFS nebo BFS
	assert(N>=40 && M>=40);

	vector<Edge> E;

	//cesta
	int D=N/2;
	int n=D+1;//pocet vrcholu
	for(int i=0; i<D; ++i){
		E.push_back({i, i+1});	
	}

	//pridavani ocasku
	vector<int> endpoints;
	while(n+15 < N && E.size()+15 < M){
		E.push_back({n, n+1});	
		E.push_back({n+1, n+2});
		E.push_back({n+2, 0});
		E.push_back({0, n+3});
		E.push_back({n+3, 1});//tohle by melo rozbit DFS/BFS 

		E.push_back({n+4, n+5});
		E.push_back({n+4, n});
		E.push_back({n+5, n});//pro jistotu at vrchol n neni list

		E.push_back({n+6, n+7});
		E.push_back({n+7, n+8});
		E.push_back({n+8, D});
		E.push_back({D, n+9});
		E.push_back({n+9, D-1});

		E.push_back({n+10, n+11});
		E.push_back({n+10, n+6});
		E.push_back({n+11, n+6});

		endpoints.push_back(n);
		endpoints.push_back(n+6);
		n+=12;
	}


	//cout << E.size() << ", " << endpoints.size() << endl << endl;

	set<pair<int,int> > F;
	while(E.size() + F.size() < M && F.size() < endpoints.size()*1ll*endpoints.size()/5){
		int u = rand() % endpoints.size();
		int v = rand() % endpoints.size();
		if(u == v) continue;
		F.insert({min(endpoints[u],endpoints[v]), max(endpoints[u],endpoints[v])});
	}



	vector<Edge> EE(F.begin(), F.end());
	EE.insert(EE.end(), E.begin(), E.end());
	

	Graph G;
	G.first = EE;
	G.second = {F.size(), n};
	return G;
}


///////////////    endvasek


void gen(int t)
{
	assert(t >= 1 && t <= 10);
	int lims[10] = {2000, 1000000, 20, 2000, 1000000, 10, 2000, 1000000, 1000000, 1000000};
	int lims2[10] = {2000, 100000, 20, 2000,  100000, 10, 2000,  100000,  100000,  100000};
	int Nlim = lims[t - 1], Nlim2 = lims2[t - 1];
	switch (t) {
	case 1:
	case 2:
		for (int i = 0; i < 2; i++)
			print(t, i, {gen_pseudotree_MeqK(Nlim / 2), gen_pseudotree_MeqK(Nlim / 2, i & 1)});
		for (int i = 2; i < 10; i++)
			print(t, i, {gen_pseudotree_MeqK(Nlim2 / 2), gen_pseudotree_MeqK(Nlim2 / 2, i & 1)});

		break;
	case 3:
	case 4:
	case 5:
		for (int i = 0; i < 3; i++)
			print(t, i, gen_K1_tree(Nlim, i == 0));

		for (int i = 3; i <= 8; i++)
			print(t, i, gen_K1_tree(Nlim2, i % 4 == 0));
		break;

	case 8:
	case 9:
	case 10:
		print(t, 9, gen_vaskuv_graph(800*1000, 1600*1000));
	case 6:
	case 7:
	
		print(t, 0, {gen_from_tree(Nlim / 2), gen_from_tree(Nlim / 2)});
		print(t, 1, gen_from_tree(Nlim));

		for (int i = 2; i <= 8; i++)
			print(t, i, {gen_from_tree(Nlim2 / 2), gen_from_tree(Nlim2 / 2, i % 4 == 0)});
		break;		
	default:
		assert(false);
	}
}

int main(void)
{

	//print(1, 1, gen_vaskuv_graph(40, 50));
	//return 0;

	for (int i = 1; i <= 10; i++)
		gen(i);
}
